import gc
import os
import json


class BrailleInputTable():
    def __init__(self, globaldata):
        self.In2Chr = ""
        self.In2Cmd = ""
        self.globaldata = globaldata

    def checkkeystr(self, keycombo):
        keycombo = keycombo.upper()
        if keycombo[0] != "B":
            return False
        if "B" in keycombo[1:]:
            return keycombo.strip()
        nl = ""
        if "1" in keycombo:
            nl += "B1"
        if "2" in keycombo:
            nl += "B2"
        if "3" in keycombo:
            nl += "B3"
        if "4" in keycombo:
            nl += "B4"
        if "5" in keycombo:
            nl += "B5"
        if "6" in keycombo:
            nl += "B6"
        if "7" in keycombo:
            nl += "B7"
        if "8" in keycombo:
            nl += "B8"
        if "L" in keycombo:
            nl += "BL"
        if "9" in keycombo:
            nl += "B9"
        if "0" in keycombo:
            nl += "B0"
        if "R" in keycombo:
            nl += "BR"
        return nl

    def combo2hex(self, combo):
        hexval = 0
        if "B1" in combo:
            hexval += 0x0001
        if "B2" in combo:
            hexval += 0x0002
        if "B3" in combo:
            hexval += 0x0004
        if "B4" in combo:
            hexval += 0x0008
        if "B5" in combo:
            hexval += 0x0010
        if "B6" in combo:
            hexval += 0x0020
        if "B7" in combo:
            hexval += 0x0040
        if "B8" in combo:
            hexval += 0x0080
        if "B9" in combo:
            hexval += 0x0100
        if "B0" in combo:
            hexval += 0x0200
        if "BL" in combo:
            hexval += 0x0400
        if "BR" in combo:
            hexval += 0x0800
        return hexval

    def hex2combo(self, hexval):
        try:
            combo = ""
            if hexval & 0x0400:
                combo = combo + "L"
            if hexval & 0x0800:
                combo = combo + "R"
            if hexval & 0x0200:
                combo = combo + "0"
            if hexval & 0x0100:
                combo = combo + "9"
            if hexval & 0x0001:
                combo = combo + "1"
            if hexval & 0x0002:
                combo = combo + "2"
            if hexval & 0x0004:
                combo = combo + "3"
            if hexval & 0x0008:
                combo = combo + "4"
            if hexval & 0x0010:
                combo = combo + "5"
            if hexval & 0x0020:
                combo = combo + "6"
            if hexval & 0x0040:
                combo = combo + "7"
            if hexval & 0x0080:
                combo = combo + "8"
        except Exception as e:
                print("combo error:", str(e), hexval)
        return "B"+combo

    def getcommand(self, brlstr):
        i = 0
        # print("getcommand", type(brlstr), brlstr)
        if not (isinstance(self.In2Chr, str) and isinstance(self.In2Cmd, str)):
            print("invalid Input Tables !")
            return None
        st = "#" + str(brlstr) + ":"
        # print("looking for", st, "in cmd")
        fp = self.In2Cmd.find(st)
        if fp >= 0:
            # print(fp)
            sp = self.In2Cmd.find(":", fp + 1) + 1
            ep = self.In2Cmd.find("#", fp + 1)
            # print(sp, ep)
            outstr = self.In2Cmd[sp:ep]
            if outstr.isdigit():
                outstr = chr(int(outstr))
        else:
            # print("looking for", st, "in chr")
            fp = self.In2Chr.find(st)
            if fp >= 0:
                sp = self.In2Chr.find(":", fp+1)+1
                ep = self.In2Chr.find("#", fp+1)
                # print(sp, ep)
                outstr = self.In2Chr[sp:ep]
                if outstr.isdigit():
                    outstr = chr(int(outstr))
                # print(type(outstr), outstr)
            else:
                print("Notfound", end="")
                return None
        return outstr

    def load_table(self, directory, name):
        gc.collect()
        longprint = False
        print("load_table", name, gc.mem_free())
        if len(directory) <= 1:
            path = directory
        else:
            path = directory[0:-1]
        print(path)
        dirlist = os.listdir(path)
        fn = ""
        for filename in dirlist:
            if (name in filename.lower()) and (".dat" in filename.lower()):
                fn = filename
        if fn == "":
            print("No filename found!")
            return False
        fn = directory + fn
        print(fn)
        ok = True
        try:
            with open(fn, "r") as textfile:
                try:
                    return textfile.readline()
                except Exception as e:
                    print("~", gc.mem_free())
                    print("Error reading file lines", str(e))
        except Exception as e:
            print("Error opening File:", str(e), fn)
            ok = False
        return None

    def load_any_table(self, directory, filename):
        d = self.load_table(directory, filename)
        if not d:
            d = self.load_table("\\", filename)
        return d

    def load_input_table(self):
        self.In2Chr = ""
        self.In2Chr = self.load_any_table(self.globaldata.getkbi(), "brailletable")
        return self.In2Chr

    def load_binding_table(self):
        self.In2Cmd = ""
        self.In2Cmd = self.load_any_table(self.globaldata.getkbc(), "keybindings")
        return self.In2Cmd


# brailleinputtable = BrailleInputTable()
brailleinputtable = None
def init(globaldata):
    global brailleinputtable
    brailleinputtable = BrailleInputTable(globaldata)

